@@ -28,7 +28,7 @@
 #include <ola/web/JsonWriter.h>
 
 #ifdef _WIN32
-#include <Winsock2.h>
+#include <ola/win/CleanWinSock2.h>
 #endif
 
 #include <fstream>
@@ -73,6 +73,7 @@ const char HTTPServer::CONTENT_TYPE_GIF[] = "image/gif";
 const char HTTPServer::CONTENT_TYPE_PNG[] = "image/png";
 const char HTTPServer::CONTENT_TYPE_CSS[] = "text/css";
 const char HTTPServer::CONTENT_TYPE_JS[] = "text/javascript";
+const char HTTPServer::CONTENT_TYPE_OCT[] = "application/octet-stream";
 
 
 /**
@@ -83,14 +84,13 @@ const char HTTPServer::CONTENT_TYPE_JS[] = "text/javascript";
  * @param key the header name
  * @param value the header value
  */
-static int AddHeaders(void *cls, enum MHD_ValueKind kind, const char *key,
-                      const char *value) {
+static int AddHeaders(void *cls, OLA_UNUSED enum MHD_ValueKind kind,
+                      const char *key, const char *value) {
   HTTPRequest *request = static_cast<HTTPRequest*>(cls);
   string key_string = key;
   string value_string = value;
   request->AddHeader(key, value);
   return MHD_YES;
-  (void) kind;
 }
 
 
@@ -106,21 +106,16 @@ static int AddHeaders(void *cls, enum MHD_ValueKind kind, const char *key,
  * @param off the offset of the data
  * @param size the number of bytes available
  */
-int IteratePost(void *request_cls, enum MHD_ValueKind kind, const char *key,
-                const char *filename, const char *content_type,
-                const char *transfer_encoding, const char *data, uint64_t off,
-                size_t size) {
+int IteratePost(void *request_cls, OLA_UNUSED enum MHD_ValueKind kind,
+                const char *key, OLA_UNUSED const char *filename,
+                OLA_UNUSED const char *content_type,
+                OLA_UNUSED const char *transfer_encoding, const char *data,
+                OLA_UNUSED uint64_t off, OLA_UNUSED size_t size) {
   // libmicrohttpd has a bug where the size isn't set correctly.
   HTTPRequest *request = static_cast<HTTPRequest*>(request_cls);
   string value(data);
   request->AddPostParameter(key, value);
   return MHD_YES;
-  (void) content_type;
-  (void) filename;
-  (void) kind;
-  (void) transfer_encoding;
-  (void) off;
-  (void) size;
 }
 
 
@@ -302,7 +297,7 @@ const string HTTPRequest::GetHeader(const string &key) const {
 const string HTTPRequest::GetParameter(const string &key) const {
   const char *value = MHD_lookup_connection_value(m_connection,
                                                   MHD_GET_ARGUMENT_KIND,
-                                                  key.data());
+                                                  key.c_str());
   if (value)
     return string(value);
   else
@@ -317,7 +312,7 @@ const string HTTPRequest::GetParameter(const string &key) const {
 bool HTTPRequest::CheckParameterExists(const string &key) const {
   const char *value = MHD_lookup_connection_value(m_connection,
                                                   MHD_GET_ARGUMENT_KIND,
-                                                  key.data());
+                                                  key.c_str());
   if (value != NULL) {
     return true;
   } else {
@@ -425,11 +420,16 @@ int HTTPResponse::Send() {
  * @param options the configuration options for the server
  */
 HTTPServer::HTTPServer(const HTTPServerOptions &options)
-    : Thread(),
+    : Thread(Thread::Options("http")),
       m_httpd(NULL),
       m_default_handler(NULL),
       m_port(options.port),
       m_data_dir(options.data_dir) {
+  ola::io::SelectServer::Options ss_options;
+  // See issue #761. epoll/kqueue can't be used with the current
+  // implementation.
+  ss_options.force_select = true;
+  m_select_server.reset(new ola::io::SelectServer(NULL, NULL, ss_options));
 }
 
 
@@ -476,8 +476,9 @@ bool HTTPServer::Init() {
                              NULL,
                              MHD_OPTION_END);
 
-  if (m_httpd)
-    m_select_server.RunInLoop(NewCallback(this, &HTTPServer::UpdateSockets));
+  if (m_httpd) {
+    m_select_server->RunInLoop(NewCallback(this, &HTTPServer::UpdateSockets));
+  }
 
   return m_httpd ? true : false;
 }
@@ -494,9 +495,15 @@ void *HTTPServer::Run() {
 
   OLA_INFO << "HTTP Server started on port " << m_port;
 
+#ifdef _WIN32
+  // set a short poll interval since we'd block too long otherwise.
+  // TODO(Lukas) investigate why the poller does not wake up on HTTP requests.
+  m_select_server->SetDefaultInterval(TimeInterval(1, 0));
+#else
   // set a long poll interval so we don't spin
-  m_select_server.SetDefaultInterval(TimeInterval(60, 0));
-  m_select_server.Run();
+  m_select_server->SetDefaultInterval(TimeInterval(60, 0));
+#endif
+  m_select_server->Run();
 
   // clean up any remaining sockets
   SocketSet::iterator iter = m_sockets.begin();
@@ -514,7 +521,7 @@ void *HTTPServer::Run() {
 void HTTPServer::Stop() {
   if (IsRunning()) {
     OLA_INFO << "Notifying HTTP server thread to stop";
-    m_select_server.Terminate();
+    m_select_server->Terminate();
     OLA_INFO << "Waiting for HTTP server thread to exit";
     Join();
     OLA_INFO << "HTTP server thread exited";
@@ -563,18 +570,18 @@ void HTTPServer::UpdateSockets() {
     } else if (ola::io::ToFD(state->descriptor->ReadDescriptor()) == i) {
       // Check if this socket must be updated.
       if (FD_ISSET(i, &r_set) && state->read == 0) {
-        m_select_server.AddReadDescriptor(state->descriptor);
+        m_select_server->AddReadDescriptor(state->descriptor);
         state->read = 1;
       } else if ((!FD_ISSET(i, &r_set)) && state->read == 1) {
-        m_select_server.RemoveReadDescriptor(state->descriptor);
+        m_select_server->RemoveReadDescriptor(state->descriptor);
         state->read = 0;
       }
 
       if (FD_ISSET(i, &w_set) && state->write == 0) {
-        m_select_server.AddWriteDescriptor(state->descriptor);
+        m_select_server->AddWriteDescriptor(state->descriptor);
         state->write = 1;
       } else if ((!FD_ISSET(i, &w_set)) && state->write == 1) {
-        m_select_server.RemoveWriteDescriptor(state->descriptor);
+        m_select_server->RemoveWriteDescriptor(state->descriptor);
         state->write = 0;
       }
       iter++;
@@ -668,8 +675,6 @@ bool HTTPServer::RegisterFile(const std::string &path,
 bool HTTPServer::RegisterFile(const std::string &path,
                               const std::string &file,
                               const std::string &content_type) {
-  // TODO(Peter): The file detail probably needs slashes swapping on Windows
-  // here or above.
   map<string, static_file_info>::const_iterator file_iter = (
       m_static_content.find(path));
 
@@ -781,9 +786,8 @@ int HTTPServer::ServeStaticContent(static_file_info *file_info,
   unsigned int length;
   string file_path = m_data_dir;
   file_path.push_back(ola::file::PATH_SEPARATOR);
-  // TODO(Peter): The below line may need fixing to swap slashes on Windows
   file_path.append(file_info->file_path);
-  ifstream i_stream(file_path.data(), ifstream::binary);
+  ifstream i_stream(file_path.c_str(), ifstream::binary);
 
   if (!i_stream.is_open()) {
     OLA_WARN << "Missing file: " << file_path;
@@ -808,7 +812,7 @@ int HTTPServer::ServeStaticContent(static_file_info *file_info,
   if (!file_info->content_type.empty())
     MHD_add_response_header(mhd_response,
                             MHD_HTTP_HEADER_CONTENT_TYPE,
-                            file_info->content_type.data());
+                            file_info->content_type.c_str());
 
   int ret = MHD_queue_response(response->Connection(),
                                MHD_HTTP_OK,
@@ -830,13 +834,13 @@ void HTTPServer::InsertSocket(bool is_readable, bool is_writeable, int fd) {
   DescriptorState *state = new DescriptorState(socket);
 
   if (is_readable) {
-    m_select_server.AddReadDescriptor(state->descriptor);
+    m_select_server->AddReadDescriptor(state->descriptor);
     state->read = 1;
   }
 
   if (is_writeable) {
     state->write = 1;
-    m_select_server.AddWriteDescriptor(state->descriptor);
+    m_select_server->AddWriteDescriptor(state->descriptor);
   }
 
   m_sockets.insert(state);
@@ -844,11 +848,11 @@ void HTTPServer::InsertSocket(bool is_readable, bool is_writeable, int fd) {
 
 void HTTPServer::FreeSocket(DescriptorState *state) {
   if (state->read) {
-    m_select_server.RemoveReadDescriptor(state->descriptor);
+    m_select_server->RemoveReadDescriptor(state->descriptor);
   }
 
   if (state->write) {
-    m_select_server.RemoveWriteDescriptor(state->descriptor);
+    m_select_server->RemoveWriteDescriptor(state->descriptor);
   }
   delete state->descriptor;
   delete state;
