@@ -52,22 +52,36 @@ namespace io {
  * @snippet udp_server.cpp UDP Server
  *
  * The SelectServer has a number of different implementations depending on the
- * platform. On systems with epoll, the flag --use-epoll will use epoll()
- * rather than select(). The PollerInterface defines the contract between the
- * SelectServer and the lower level, platform dependant Poller classes.
+ * platform. On systems with epoll, the flag --no-use-epoll will disable the
+ * use of epoll(), reverting to select(). The PollerInterface defines the
+ * contract between the SelectServer and the lower level, platform dependant
+ * Poller classes.
  *
  * All methods except Execute() and Terminate() must be called from the thread
  * that Run() was called in.
  */
 class SelectServer: public SelectServerInterface {
  public :
+  struct Options {
+   public:
+    Options(): force_select(false) {}
+
+    /**
+     * @brief Fall back to the select() implementation even if the flags are
+     * set for kqueue/epoll.
+     */
+    bool force_select;
+  };
+
   /**
    * @brief Create a new SelectServer
    * @param export_map the ExportMap to use for stats
-   * @param clock the Clock to use to keep time
+   * @param clock the Clock to use to keep time.
+   * @param options Additional options
    */
   SelectServer(ola::ExportMap *export_map = NULL,
-               Clock *clock = NULL);
+               Clock *clock = NULL,
+               const Options &options = Options());
 
   /**
    * @brief Clean up.
